package cz.cvut.kbss.reporting.service.repository;

import cz.cvut.kbss.reporting.exception.AuthorizationException;
import cz.cvut.kbss.reporting.exception.UsernameExistsException;
import cz.cvut.kbss.reporting.model.Person;
import cz.cvut.kbss.reporting.persistence.dao.GenericDao;
import cz.cvut.kbss.reporting.persistence.dao.PersonDao;
import cz.cvut.kbss.reporting.service.security.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.Objects;

@Service
public class PersonService extends AbstractRepositoryService<Person> {

    private final PersonDao dao;

    private final PasswordEncoder passwordEncoder;

    private final SecurityUtils securityUtils;

    @Autowired
    public PersonService(PersonDao dao, PasswordEncoder passwordEncoder, SecurityUtils securityUtils) {
        this.dao = dao;
        this.passwordEncoder = passwordEncoder;
        this.securityUtils = securityUtils;
    }

    @Override
    protected GenericDao<Person> getPrimaryDao() {
        return dao;
    }

    @Override
    void prePersist(Person instance) {
        instance.encodePassword(passwordEncoder);
    }

    @Override
    void preUpdate(Person instance) {
        final Person current = securityUtils.getCurrentUser();
        if (!current.getId().equals(instance.getId())) {
            throw new AuthorizationException("Modifying other user\'s account is forbidden.");
        }
        verifyUniqueUsername(instance);
        if (instance.getPassword() != null) {
            instance.encodePassword(passwordEncoder);
        } else {
            instance.setPassword(current.getPassword());
        }
    }

    private void verifyUniqueUsername(Person update) {
        final Person existing = dao.findByUsername(update.getUsername());
        if (existing != null && !existing.getId().equals(update.getId())) {
            throw new UsernameExistsException("Username " + update.getUsername() + " already exists.");
        }
    }

    @Override
    void postUpdate() {
        securityUtils.updateCurrentUser();
    }

    /**
     * Checks whether an instance with the specified username exists.
     *
     * @param username The username to search for
     * @return Record existence status
     */
    public boolean exists(String username) {
        Objects.requireNonNull(username);
        return dao.exists(username);
    }

    public Person findByUsername(final String username) {
        return dao.findByUsername(username);
    }
}
