'use strict';

import React from "react";
import Reflux from "reflux";
import {Label, Panel, Table} from "react-bootstrap";
import PropTypes from "prop-types";
import Actions from "../../actions/Actions";
import CollapsibleText from "../CollapsibleText";
import I18nWrapper from "../../i18n/I18nWrapper";
import injectIntl from "../../utils/injectIntl";
import Mask from "../Mask";
import ReportStore from "../../stores/ReportStore";
import Utils from "../../utils/Utils";

const RECENTLY_EDITED_COUNT = 10;

class RecentlyEditedReports extends Reflux.Component {

    constructor(props) {
        super(props);
        this.i18n = props.i18n;
        this.state = {};
        this.store = ReportStore;
    }

    componentWillMount() {
        super.componentWillMount();
        Actions.loadAllReports();
    }

    _filterRecentReports() {
        const reports = this.state.reports.slice();
        reports.sort(function (a, b) {
            const aEdited = a.lastModified ? a.lastModified : a.dateCreated,
                bEdited = b.lastModified ? b.lastModified : b.dateCreated;
            return bEdited - aEdited;
        });
        return reports.slice(0, RECENTLY_EDITED_COUNT);
    }

    render() {
        const title = <h5>{this.i18n('dashboard.recent-panel-heading')}</h5>;
        return <Panel header={title} bsStyle='info' style={{height: '100%'}}>
            {this._renderPanelContent()}
        </Panel>;
    }

    _renderPanelContent() {
        if (!this.state.reports) {
            return <Mask text={this.i18n('reports.loading-mask')} classes='mask-container'/>;
        }
        const recentReports = this._renderRecentReports(this._filterRecentReports());
        if (recentReports.length > 0) {
            return <Table striped bordered condensed hover>
                <thead>
                <tr>
                    <th className='col-xs-5'>{this.i18n('headline')}</th>
                    <th className='col-xs-3 content-center' title={this.i18n('reports.table-date.tooltip')}>
                        {this.i18n('reports.table-date')}
                    </th>
                    <th className='col-xs-3 content-center'>{this.i18n('dashboard.recent-table-last-edited')}</th>
                    <th className='col-xs-1 content-center'>{this.i18n('reports.table-type')}</th>
                </tr>
                </thead>
                <tbody>
                {recentReports}
                </tbody>
            </Table>;
        } else {
            return <div>{this.i18n('dashboard.recent.no-reports')}</div>;
        }
    }

    _renderRecentReports(reports) {
        const toRender = [];
        for (let i = 0, len = reports.length; i < len; i++) {
            toRender.push(<ReportRow key={reports[i].id} report={reports[i]} openReport={this.props.openReport}/>);
        }
        return toRender;
    }
}

RecentlyEditedReports.propTypes = {
    openReport: PropTypes.func.isRequired
};

class ReportRow extends React.Component {
    constructor(props) {
        super(props);
        this.i18n = props.i18n;
    }

    onOpenClick = (e) => {
        e.preventDefault();
        this.props.openReport(this.props.report);
    };

    render() {
        const report = this.props.report,
            dateEdited = report.lastModified ? report.lastModified : report.created;
        return <tr>
            <td className='report-row'>
                <a href='javascript:void(0);' onClick={this.onOpenClick} title={this.i18n('reports.open-tooltip')}>
                    <CollapsibleText text={report.occurrence.name} maxLength={20}/>
                </a>
            </td>
            <td className='content-center report-row'>{Utils.formatDate(report.occurrence.startTime)}</td>
            <td className='content-center report-row'>{Utils.formatDate(dateEdited)}</td>
            <td className='content-center report-row'>
                <Label title={this.i18n('occurrencereport.title')}>{this.i18n('occurrencereport.label')}</Label>
            </td>
        </tr>;
    }
}

ReportRow.propTypes = {
    report: PropTypes.object.isRequired,
    openReport: PropTypes.func.isRequired
};

ReportRow = injectIntl(I18nWrapper(ReportRow));

export default injectIntl(I18nWrapper(RecentlyEditedReports));
