'use strict';

import React from "react";
import {Alert, Button, Panel} from "react-bootstrap";
import Ajax from "../../utils/Ajax";
import I18nWrapper from "../../i18n/I18nWrapper";
import injectIntl from "../../utils/injectIntl";
import Input from "../HorizontalInput";
import Logger from "../../utils/Logger";
import Mask from "../Mask";
import PersonValidator from "../../validator/PersonValidator";
import Routing from "../../utils/Routing";
import Routes from "../../utils/Routes";

class Register extends React.Component {

    constructor(props) {
        super(props);
        this.i18n = props.i18n;
        this.state = {
            firstName: '',
            lastName: '',
            username: '',
            password: '',
            passwordConfirm: '',
            alertVisible: false,
            errorMessage: '',
            mask: false
        };
    }

    componentWillUnmount() {
        if (this.alertTimeout) {
            clearTimeout(this.alertTimeout);
        }
    }

    onChange = (e) => {
        const change = {};
        change[e.target.name] = e.target.value;
        this.setState(change);
    };

    onKeyDown = (e) => {
        if (e.key === 'Enter') {
            this.register();
        }
    };

    showAlert = (msg) => {
        this.setState({alertVisible: true, errorMessage: msg, mask: false});
        this.alertTimeout = setTimeout(this.dismissAlert, 5000);
    };

    dismissAlert = () => {
        this.setState({alertVisible: false});
        this.alertTimeout = null;
    };

    isValid = () => {
        return PersonValidator.isValid(this.state);
    };

    register = () => {
        if (!this.isValid()) {
            return;
        }
        const userData = {
            firstName: this.state.firstName,
            lastName: this.state.lastName,
            username: this.state.username,
            password: this.state.password
        };
        Ajax.post('rest/persons', userData).end(function (body, resp) {
            if (resp.status === 201) {
                this.doSyntheticLogin(userData.username, userData.password);
            }
        }.bind(this), function (err) {
            this.showAlert(err.message ? err.message : 'Unknown error.');
        }.bind(this));
        this.setState({mask: true});
    };

    /**
     * After successful registration, perform a synthetic login so that the user receives his session and can start
     * working.
     */
    doSyntheticLogin = (username, password) => {
        Ajax.post('j_spring_security_check', null, 'form').send('username=' + username).send('password=' + password)
            .end(function (data, resp) {
                const status = JSON.parse(resp.text);
                if (!status.success || !status.loggedIn) {
                    this.showAlert();
                    return;
                }
                Routing.transitionToHome();
            }.bind(this), function (err) {
                Logger.error('Unable to perform synthetic login. Received response with status ' + err.status);
            });
    };

    cancel = () => {
        Routing.transitionTo(Routes.login);
    };

    render() {
        const panelCls = this.state.alertVisible ? 'register-panel expanded' : 'register-panel',
            mask = this.state.mask ? (<Mask text={this.i18n('register.mask')}/>) : null;

        return <Panel header={<h3>{this.i18n('register.title')}</h3>} bsStyle='info' className={panelCls}>
            {mask}
            <form className='form-horizontal' style={{margin: '0.5em 0 0 0'}}>
                {this.renderAlert()}
                <div className='row'>
                    <div className='col-xs-6'>
                        <Input type='text' name='firstName' label={this.i18n('register.firstName') + '*'}
                               value={this.state.firstName}
                               labelWidth={4} inputWidth={8} onChange={this.onChange}/>
                    </div>
                    <div className='col-xs-6'>
                        <Input type='text' name='lastName' label={this.i18n('register.lastName') + '*'}
                               value={this.state.lastName}
                               labelWidth={4} inputWidth={8} onChange={this.onChange}/>
                    </div>
                </div>
                <div className='row'>
                    <div className='col-xs-6'>
                        <Input type='text' name='username' label={this.i18n('register.username') + '*'}
                               value={this.state.username}
                               labelWidth={4} inputWidth={8} onChange={this.onChange}/>
                    </div>
                </div>
                <div className='row'>
                    <div className='col-xs-6'>
                        <Input type='password' name='password' label={this.i18n('register.password') + '*'}
                               labelWidth={4} inputWidth={8}
                               onChange={this.onChange} value={this.state.password}/>
                    </div>
                    <div className='col-xs-6'>
                        {this.renderPasswordConfirm()}
                    </div>
                </div>
                <div style={{margin: '1em 0em 0em 0em', textAlign: 'center'}}>
                    <Button bsStyle='success' bsSize='small' ref='submit'
                            disabled={!this.isValid() || this.state.mask}
                            onClick={this.register}>{this.i18n('register.submit')}</Button>
                    <Button bsSize='small' onClick={this.cancel} style={{margin: '0 0 0 3.2em'}}
                            disabled={this.state.mask}>{this.i18n('cancel')}</Button>
                </div>
            </form>
        </Panel>;
    }

    renderAlert() {
        return this.state.alertVisible ?
            <Alert bsStyle='danger' bsSize='small' onDismiss={this.dismissAlert}>
                <div>{this.state.errorMessage}</div>
            </Alert> : null;
    }

    renderPasswordConfirm() {
        const passwordMatch = this.state.password === this.state.passwordConfirm;
        return <Input type='password' name='passwordConfirm' label={this.i18n('register.password-confirm') + '*'}
                      labelWidth={4} inputWidth={8} onChange={this.onChange} onKeyDown={this.onKeyDown}
                      value={this.state.passwordConfirm} validation={!passwordMatch ? 'error' : null}
                      title={!passwordMatch ? this.i18n('register.passwords-not-matching-tooltip') : null}/>;
    }
}

export default injectIntl(I18nWrapper(Register));
