'use strict';

var React = require('react');
var ReactDOM = require('react-dom');
var TestUtils = require('react-dom/test-utils');

var Actions = require('../../js/actions/Actions');
var TestApp = require('./TestApp');

module.exports = {

    /**
     * Renders the specified component, wrapping it in a TestApp instance, which is used to initialize some required
     * context data, e.g. i18n.
     * @param component Component to render
     * @return {*|!ReactComponent} The rendered component
     */
    render: function (component) {
        var type = component.type,
            result = TestUtils.renderIntoDocument(<TestApp>{component}</TestApp>),
            renderedComponent = TestUtils.findRenderedComponentWithType(result, type);
        //console.log(renderedComponent);
        if (renderedComponent.getWrappedInstance) {
            return renderedComponent.getWrappedInstance();
        } else {
            return renderedComponent;
        }
    },

    /**
     * Renders the specified component into a table body, wrapping it all in a TestApp instance, which is used to
     * initialize some required context data, e.g. i18n.
     *
     * This is useful for components representing table rows.
     * @param component Component to render
     * @return {*|!ReactComponent} The rendered component
     */
    renderIntoTable: function (component) {
        var type = component.type,
            result = TestUtils.renderIntoDocument(<TestApp>
                <table>
                    <tbody>{component}</tbody>
                </table>
            </TestApp>),
            renderedComponent = TestUtils.findRenderedComponentWithType(result, type);
        //console.log(renderedComponent);
        if (renderedComponent.getWrappedInstance) {
            return renderedComponent.getWrappedInstance();
        } else {
            return renderedComponent;
        }
    },

    /**
     * Finds component with the specified text.
     * @param root Root of the tree where the component is searched for
     * @param component Component class
     * @param text Component text
     */
    getComponentByText: function (root, component, text) {
        var components = TestUtils.scryRenderedComponentsWithType(root, component);
        return this._getNodeByText(components, text, true);
    },

    _getNodeByText: function (components, text, strict) {
        for (var i = 0, len = components.length; i < len; i++) {
            var node = ReactDOM.findDOMNode(components[i]);
            if (strict && node.textContent === text) {
                return node;
            } else if (!strict && node.textContent.indexOf(text) !== -1) {
                return node;
            }
        }
        return null;
    },

    /**
     * Finds component with the specified text.
     *
     * This version searches components by tag, so it will catch also simple components like div.
     * @param root Root of the tree where the component is searched for
     * @param tag Tag name
     * @param text Component text
     */
    getComponentByTagAndText: function (root, tag, text) {
        var components = TestUtils.scryRenderedDOMComponentsWithTag(root, tag);
        return this._getNodeByText(components, text, true);
    },

    /**
     * Finds component which contains the specified text.
     *
     * This version searches components by tag, so it will catch also simple components like div.
     * @param root Root of the tree where the component is searched for
     * @param tag Tag name
     * @param text Text contained in the component's text content
     */
    getComponentByTagAndContainedText: function (root, tag, text) {
        var components = TestUtils.scryRenderedDOMComponentsWithTag(root, tag);
        return this._getNodeByText(components, text, false);
    },

    /**
     * Creates a mock objects with the specified methods.
     *
     * The methods adhere to the builder pattern - they return the mock instance itself.
     * @param reqMockMethods array of methods to mock
     */
    mockRequestMethods: function (reqMockMethods) {
        var reqMock = jasmine.createSpyObj('request', reqMockMethods);
        for (var i = 0; i < reqMockMethods.length; i++) {
            // All mock methods just return the instance to adhere to the builder pattern implemented by request
            reqMock[reqMockMethods[i]].and.callFake(function () {
                return reqMock;
            });
        }
        return reqMock;
    },

    /**
     * Mocks the Logger, so that test console output is not polluted with log messages.
     */
    mockLogger: function () {
        return jasmine.createSpyObj('Logger', ['warn', 'log', 'error']);
    },

    /**
     * Returns true if the arrays are equal, i.e. they have the same size and contain the same elements.
     */
    arraysEqual: function (a, b) {
        if (a === b) return true;
        if (a == null || b == null) return false;
        if (a.length != b.length) return false;

        for (var i = 0; i < a.length; ++i) {
            if (a[i] !== b[i]) {
                return false;
            }
        }
        return true;
    },

    /**
     * Binds store methods directly to actions to work around asynchronous processing of actions.
     * @param actionName Name of the action
     * @param store Store, the corresponding store method is found according to the usual onActionName scheme.
     */
    bindActionsToStoreMethods: function (actionName, store) {
        var fn = 'on' + actionName.charAt(0).toUpperCase() + actionName.substring(1);
        // For some reason, it didn't suffice (in some cases) to pass the store[fn] as argument to callFake, so that's
        // why the wrapping function is here
        spyOn(Actions, actionName).and.callFake(function () {
            store[fn].apply(store, arguments);
        });
    },

    /**
     * Custom Jasmine matchers
     */
    customMatchers: {

        toBeLexGreaterOrEqual: function (util, customEqualityTesters) {
            return {
                compare: function (actual, expected) {
                    if (expected === undefined) {
                        expected = '';
                    }
                    var result = {};
                    result.pass = actual.toUpperCase().localeCompare(expected.toUpperCase()) >= 0;
                    if (result.pass) {
                        result.message = 'Expected ' + actual + ' not to be lexicographically greater or equal to ' + expected;
                    } else {
                        result.message = 'Expected ' + actual + ' to be lexicographically greater or equal to ' + expected;
                    }
                    return result;
                }
            }
        },
        toBeLexGreaterThan: function (util, customEqualityTesters) {
            return {
                compare: function (actual, expected) {
                    if (expected === undefined) {
                        expected = '';
                    }
                    var result = {};
                    result.pass = actual.toUpperCase().localeCompare(expected.toUpperCase()) > 0;
                    if (result.pass) {
                        result.message = 'Expected ' + actual + ' not to be lexicographically greater than ' + expected;
                    } else {
                        result.message = 'Expected ' + actual + ' to be lexicographically greater than ' + expected;
                    }
                    return result;
                }
            }
        }
    }
};
